/*
 * smtp - Firestorm SMTP decode plugin
 * Copyright (c) 2002 Gianni Tedesco <gianni@scaramanga.co.uk>
 * Released under the terms of the GNU GPL v2
 *
 * TODO:
 *  o Track application layer state
 *  o Parse responses and addresses
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <firestorm.h>
#include <args.h>
#include <packet.h>
#include <plugin.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <preproc.h>
#include <netinet/in.h>

PLUGIN_STD_DEFS();
static proc_dispatch dispatch;

/* Returns number of bytes fit to proces */
static size_t
smtp_sdecode(struct packet *p, char *buf, size_t len)
{
	char *end;
	size_t ret = 0;
	int i;

	if ( buf[len-1] == '\n' )
		return len;
	
	for(end=buf+len, i=0; buf < end; buf++, i++) {
		if ( *buf == '\n' )
			ret = i + 1;
	}

	return ret;
}

void smtp_response_decode(struct packet *p)
{
	char *begin;
	char *end;
	struct fvec v;
	unsigned int code;

	begin = p->layer[p->llen].h.raw;
	end = p->layer[p->llen-2].h.raw +
		ntohs(p->layer[p->llen-2].h.ip->tot_len);

	v.ptr = begin;
	v.len = end - begin;

	if ( v.len < 4 ) /* sizeof("NNN\n") == 4 */
		return;

	if ( vtouint(&v, &code) != 3 )
		return;

	dispatch(p);
}

/* SMTP decode callback */
static void smtp_decode(struct packet *p)
{
	if ( p->layer[p->llen-1].flags & FLAG_TCP_2SVR ) {
		dispatch(p);
	}else{
		smtp_response_decode(p);
	}
}

struct proto smtp_p={
	.name="smtp",
	.sdecode=smtp_sdecode,
	.decode=smtp_decode,
	.flow_len=0,
};

struct proto_req smtp_r[]={
	proto_request("tcp", __constant_htons(25)),
	null_request()
};

int PLUGIN_DECODE (struct decode_api *d)
{
	int ok=0;

	object_check(d);

	dispatch=d->dispatch;

	ok+=d->decode_add(&smtp_p, smtp_r);

	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	plugin_check(in, out);

	PLUGIN_ID("decode.smtp", "Simple Mail Transfer Protocol");
	PLUGIN_VERSION(0, 1);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@scaramanga.co.uk");
	PLUGIN_LICENSE("GPL");

	return PLUGIN_ERR_OK;
}

int PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
