#ifndef __ELOG_WRITE_HEADER_INCLUDED__
#define __ELOG_WRITE_HEADER_INCLUDED__

#include <config.h>
#include <fbuf.h>

#define ALERT_FN "alert.elog"
#define ALERT_FNLEN 23

#define SPOOL_DEFAULT_MINS 60
#define SPOOL_DEFAULT_SIZE (512*1024)
#define SPOOL_DEFAULT_BUFFER (16*1024)

/* Number of available I/O vectors */
#define ALERT_VEC 6

/* Stormwall mode */
#define STORMWALL_NONE 0 /* stormwall comms disabled */
#define STORMWALL_WAIT 1 /* wait for stormwall to come alive */
#define STORMWALL_FAIL 2 /* fail immidietly if stormwall is not ready */

struct elog_spool_head {
	struct elog_spool *next, *prev;
};

struct elog_spool {
	struct elog_spool *next, *prev;

	/* Current packet data */
	struct fvec iov[ALERT_VEC];
	struct elog_pkthdr ph;
	size_t tot_len;
	int io_len;

	/* The files */
	char *alert_fn;
	char *rotate_fn;
	char *log_dir;

	/* Output buffer */
	struct fbuf buf;
	int nobuf;

	/* state information */
	size_t size;
	struct timeval tv;

	/* rotation parameters */
	unsigned int max_bytes;
	unsigned int max_time;

	/* stormwall stuff */
	char *fifo_fn;
	int fifo_fd;
	int stormwall;
};

static inline int spool_isempty(struct elog_spool *s)
{
	return (s->tv.tv_sec) ? 0 : 1;
}

#ifndef __PLUGIN__
/* Spool API */
struct elog_spool *spool_new(void);
void spool_delete(struct elog_spool *);
void spool_init_all(void);
int spool_open(struct elog_spool *);
int spool_dump_header(struct elog_spool *);
void spool_close(struct elog_spool *);
void spool_rotate(struct elog_spool *, int);
void spool_fini(struct elog_spool *);
int spool_flush(struct elog_spool *);
int spool_set_buf(struct elog_spool *, size_t);
void spool_link(struct elog_spool *);
void spool_packet(struct alert *, struct packet *);
void spool_rotate_all(void);
#endif

#endif /* __ELOG_WRITE_HEADER_INCLUDED__ */
