#ifndef __FIRESTORM_HEADER_INCLUDED__
#define __FIRESTORM_HEADER_INCLUDED__

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Next 4 bits for type */
#define M_UNSET	0 /* Unclassified */
#define M_CRIT	1 /* Service/data is lost */
#define M_ERR	2 /* We can't do something you asked for */
#define M_WARN	3 /* We can work around this, but you should know */
#define M_INFO	4 /* Informational notices */
#define M_DEBUG	5 /* For developers */
#define M_MAX	6

#ifndef __PLUGIN__
void mesg(unsigned char, char *, ...);
void mesg_init(void);
void mesg_logfile(char *);
#else
typedef void (*proc_mesg)(unsigned char, char *, ...);
#endif

/* Generic vector type, used all over */
struct fvec {
	void *ptr;
	size_t len;
};

/* Convert a vector string in to an unsigned int */
static inline int
vtouint(struct fvec *v, unsigned int *u)
{
	unsigned char *chr = v->ptr;
	int i;

	*u = 0;

	for(i=0; i<v->len; i++) {
		unsigned int digit = *chr++ - '0';
		if ( digit >= 10 )
			break;

		*u *= 10;
		*u += digit;
	}

	return i;
}

/* Case-insensitive comparison of two vectors */
static inline int
vcasecmp(const struct fvec *v1, const struct fvec *v2)
{
	const char *end1, *end2;
	const char *p1, *p2;

	end1 = v1->ptr + v1->len;
	end2 = v2->ptr + v2->len;

	p1 = v1->ptr;
	p2 = v2->ptr;

	for(;;) {
		if ( p1==end1 && p2==end2 )
			return 0;

		if ( p1==end1 )
			return -1;

		if ( p2==end2 )
			return 1;

		if ( toupper(*p1) < toupper(*p2) )
			return -1;

		if ( toupper(*p1) > toupper(*p2) )
			return 1;

		p1++; p2++;
	}
	
}

/* Case-insensitive comparison of a string and a vector */
static inline int
vstrcmp(const struct fvec *v1, const char *str)
{
	struct fvec v2;

	v2.ptr = (void *)str;
	v2.len = strlen(str);

	return vcasecmp(v1, &v2);
}

#endif /* __FIRESTORM_HEADER_INCLUDED__ */
