#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <firestorm.h>
#include <packet.h>
#include <alert.h>
#include <signature.h>
#include <matcher.h>
#include <decode.h>
#include <plugin.h>
#include <pkt/ipx.h>

PLUGIN_STD_DEFS();

proc_template template_shortrange;

int ipxaddr_compare(void *p1, void *p2)
{
	return memcmp(p1,p2,6) ? CMP_DIFFER : CMP_EQUAL;
}

int ipx_dst_match(struct packet *pkt, void *priv, unsigned int l, int n)
{
	return n^(memcmp(priv,pkt->layer[l].h.ipx->dst.node,6)==0);
}

int ipx_src_match(struct packet *pkt, void *priv, unsigned int l, int n)
{
	return n^(memcmp(priv,pkt->layer[l].h.ipx->src.node,6)==0);
}

int ipx_type_match(struct packet *pkt, void *priv, unsigned int l, int n)
{
	struct shortrange *p=(struct shortrange *)priv;
	u_int16_t port=pkt->layer[l].h.ipx->type;
	return n^(port>=p->min && port<=p->max);
}
int ipx_dstsock_match(struct packet *pkt, void *priv, unsigned int l, int n)
{
	struct shortrange *p=(struct shortrange *)priv;
	u_int16_t sock=pkt->layer[l].h.ipx->dst.sock;
	return n^(sock>=p->min && sock<=p->max);
}

int ipx_srcsock_match(struct packet *pkt, void *priv, unsigned int l, int n)
{
	struct shortrange *p=(struct shortrange *)priv;
	u_int16_t sock=pkt->layer[l].h.ipx->src.sock;
	return n^(sock>=p->min && sock<=p->max);	
}

int ipx_addr_validate(char *args, void **priv,
			struct criteria *m, u_int32_t *c)
{
	unsigned char buf[IPX_NODE_LEN];
	int i;

	if ( args == NULL )
		return 0;

	for(i=0; i<sizeof(buf);) {
		unsigned long int l;
		char *end;

		l=strtoul(args, &end, 16);
		if ( end == args )
			return 0;

		if ( l & ~0xffUL )
			return 0;

		buf[i++] == l;

		if ( *end == '\0' ) {
			if ( i != sizeof(buf) )
				return 0;
			break;
		}

		if ( *end != ':' )
			return 0;

		if ( i >= sizeof(buf) )
			return 0;

		args = end + 1;
	}

	if ( *priv=malloc(sizeof(buf)) )
		return 0;

	memcpy((u_int8_t *)*priv, buf, sizeof(buf));
	return 1;
}

proc_match_match
ipx_dst_validate(char *args, void **priv,
		struct criteria *m, u_int32_t *c)
{
	return ipx_addr_validate(args,priv,m,c) ? ipx_dst_match : NULL;
}
	
proc_match_match
ipx_src_validate(char *args, void **priv,
		struct criteria *m, u_int32_t *c)
{
	return ipx_addr_validate(args,priv,m,c) ? ipx_src_match : NULL;
}
	
proc_match_match
ipx_type_validate(char *args, void **priv,
			struct criteria *m, u_int32_t *c)
{
	/* the template_shortrange function handles args such as 0x04:0x10
	 * It returns a struct shortrange containing the min and max values
	 * which are handled in the matcher function
	*/
	return ( template_shortrange(args, priv) ) ? ipx_type_match : NULL;
}
	
proc_match_match
ipx_dstsock_validate(char *args, void **priv,
			struct criteria *m, u_int32_t *c)
{
	return ( template_shortrange(args, priv) ) ? ipx_dstsock_match : NULL;
}
	
proc_match_match
ipx_srcsock_validate(char *args, void **priv,
			struct criteria *m, u_int32_t *c)
{
	return ( template_shortrange(args, priv) ) ? ipx_srcsock_match : NULL;
}
	
struct matcher ipx_matchers[]={
	matcher_init("ipx_dst", MCOST_NET, ipx_dst_validate, ipxaddr_compare, MATCHER_FREE),
	matcher_init("ipx_src", MCOST_NET, ipx_src_validate, ipxaddr_compare, MATCHER_FREE),
	matcher_init("ipxtype", MCOST_NET, ipx_type_validate, MCMP_LR, MATCHER_FREE),
	matcher_init("ipxdstsock", MCOST_NET, ipx_dstsock_validate, MCMP_LR, MATCHER_FREE),
	matcher_init("ipxsrcsock", MCOST_NET, ipx_srcsock_validate, MCMP_LR, MATCHER_FREE),
	matcher_null()
};

int PLUGIN_MATCHER (struct matcher_api *m)
{
	object_check(m);
	
	template_shortrange=m->template_shortrange;
	if ( !m->matcher_add(ipx_matchers) )
		return PLUGIN_ERR_FAIL;
	return PLUGIN_ERR_OK;
}


int PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{	
	plugin_check(in, out);
	
	PLUGIN_ID("match.ipx", "IPX matching routines");
	PLUGIN_VERSION(1, 1);
	PLUGIN_AUTHOR("John Leach", "john@johnleach.co.uk");
	PLUGIN_LICENSE("GPL");

	return PLUGIN_ERR_OK;
}

int PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
