/*
* This file is part of Firestorm NIDS
* Copyright (c) 2003 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*
* This is the GNOME firestorm analyst console. Most of the GUI code
* is actually in src/g_*.c
*/
#include <gnome.h>

#include <firestorm.h>
#include <args.h>
#include <packet.h>
#include <cleanup.h>
#include <loader.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>

GtkWidget *create_mainwin(void);

/* Functions we may need to fake */
int return_one(void) {return 1;}
void return_void(void) {return;}
void *return_null(void) {return NULL;}
void serial_number(serial_t *s){*s=0;}

struct {
	const char *name;
	void *data;
}exports[]={
	/* Generic */
	{"mesg",		mesg},
	{"args.parse", 		args_parse},
	{"serial_number",	serial_number},

	/* Object lookup */
	{"decode.subproto", 	decode_subproto},
	{"decode.proto", 	decode_proto},
	{"matcher.find",	return_null},

	/* Needed for signature engines */
	{"alert",		return_void},
	{"generator.add",	return_one},
	{"detect.add_sig",	return_one},
	{"detect.optimize",	return_void},
	{"detect.free_sig",	return_void},
	{"detect",		return_void},

	{NULL,NULL}
};

void *plugin_callback(const char *item)
{
	u_int32_t i;

	for(i=0; exports[i].name; i++) {
		if ( !strcmp(exports[i].name, item) ) {
			return exports[i].data;
		}
	}

	fprintf(stderr, "resource: request for unknown object: '%s'\n", item);
	return NULL;
}

int main(int argc, char **argv)
{
	GtkWidget *mainwin;

	gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE,
		argc, argv, GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR, NULL);
	mainwin = create_mainwin();

	loader_init(plugin_callback);

#if 1
	loader_load_dir(PLUGIN_DIR "/protocols");
#else
	loader_load_dir("./decode_plugins/.libs/");
#endif

	decode_load();

	/* Initialise gnome stuff */
	gtk_widget_show_all(mainwin);

	gtk_main();
	return 0;
}
