#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <ctype.h>
#include <unistd.h>
#include <time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <syslog.h>

#include <firestorm.h>
#include <packet.h>
#include <args.h>
#include <cleanup.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <target.h>
#include <plugin.h>
#include <capture.h>

PLUGIN_STD_DEFS();

static char *state_str[]={
	"CLOSED",
	"ESTABLISHED",
	"SYN_SENT",
	"SYN_RECV",
	"FIN_WAIT1",
	"FIN_WAIT2",
	"TCP_TIME_WAIT",
	"TCP_CLOSE",
	"TCP_CLOSE_WAIT",
	"TCP_LAST_ACK",
	"TCP_LISTEN",
	"TCP_CLOSING"
};

/* Imported functions */
proc_args_parse args_parse;
proc_decode_proto decode_proto;

struct proto *ipproto=NULL;
struct proto *http_proto=NULL;

/* Private data structure */
struct syslog_priv {
	char *fn;
	char *buf;
	size_t buflen;
};

void log_http(char *buf, int len, struct http_request *h)
{
	if ( h->type != HTTP_REQUEST )
		return;

	/* 28 = 14+12+2 = extraneous bits of string */
	if ( len < 28 + h->uri.len + h->method.len )
		return;

	memcpy(buf, " http_method='", 14); buf+=14;
	memcpy(buf, h->method.ptr, h->method.len); buf+=h->method.len;
	memcpy(buf, "' http_uri='", 12); buf+=12;
	memcpy(buf, h->uri.ptr, h->uri.len); buf+=h->uri.len;
	memcpy(buf, "'", 2);
}

/* Configuration callbacks */
int cb_file(struct arg *a, void *priv)
{
	struct syslog_priv *p=priv;
	return (p->fn=strdup(a->val.v_str)) ? 1 : 0;
}

int syslog_a(struct alert *a,
	struct packet *pkt,
	void *priv)
{
	union {
		struct pkt_tcphdr *tcph;
		struct pkt_udphdr *udph;
		struct pkt_icmphdr *icmph;
	}u;
	u_int32_t flags;
	struct syslog_priv *p=priv;
	struct pkt_iphdr *iph;
	char sip[16], dip[16];
	static char pbuf[4096];
	struct in_addr ina;
	unsigned int l;
	int ret;

	/* TCP stuff */
	char tcpflags[]="FSRPAU12";
	char disp[9];
	int n, x;
	struct tcp_session *s;
	int s_state, c_state;

	if ( !pkt ) return 0;

	for(l=0; l<pkt->llen; l++) {
		if ( pkt->layer[l].proto==ipproto ) goto found;
	}

	mesg(M_CRIT,"syslog: No IP header found");
	return 1;

found:
	iph=pkt->layer[l].h.ip;

	if ( l+1 >= pkt->llen ) {
		pbuf[0]=0;
		goto just_ip;
	}

	u.tcph=pkt->layer[l+1].h.tcp;
	flags=pkt->layer[l+1].flags;

	/* Convert IPs to strings */
	ina.s_addr=iph->saddr;
	strncpy(sip, inet_ntoa(ina), 16);
	sip[15]=0;
	ina.s_addr=iph->daddr;
	strncpy(dip, inet_ntoa(ina), 16);
	dip[15]=0;

	switch(iph->protocol) {
	case 6:
		for(n=0,x=1; n<8; n++, x*=2) {
			if ( u.tcph->flags.flags & x ){
				disp[n]=tcpflags[n];
			}else{
				disp[n]='*';
			}
		}disp[8]='\0';

		if ( (s=pkt->layer[l+1].session) ) {
			s_state=s->server.state;
			c_state=s->client.state;
		}else{
			s_state=0;
			c_state=0;
		}

		ret=snprintf(pbuf, sizeof(pbuf), "spt=%u dpt=%u flags=%s"
			"from=%s server=%s client=%s",
			ntohs(u.tcph->sport)&0xffff,
			ntohs(u.tcph->dport)&0xffff,
			disp,
			(flags&FLAG_TCP_2SVR) ? "client" : "server",
			state_str[s_state],
			state_str[c_state]);

		if ( ret<0 ) {
			pbuf[0]=0;
			break;
		}

		if ( l+2<pkt->llen &&
			pkt->layer[l+2].proto==http_proto
			&& pkt->layer[l+2].session ) {
			log_http(pbuf+ret, sizeof(pbuf)-ret, pkt->layer[l+2].session);
		}

		break;
	case 17:
		snprintf(pbuf, sizeof(pbuf), "spt=%u dpt=%u",
			ntohs(u.udph->sport)&0xffff,
			ntohs(u.udph->dport)&0xffff);
		break;
	case 1:
		snprintf(pbuf, sizeof(pbuf), "type=%u code=%u",
			u.icmph->type&0xff,
			u.icmph->code&0xff);
		break;
	default:
		pbuf[0]=0;
	}

just_ip:
	ret=snprintf(p->buf, p->buflen,
		"%lu.%.6lu alert=%s sig=%u.%u priority=%u "
		"src=%s dst=%s proto=%u %s : %s",
		pkt->time.tv_sec,
		pkt->time.tv_usec,
		a->gen->name,
		a->sid, a->rev,
		a->priority,
		sip, dip,
		iph->protocol&0xff,
		pbuf,
		a->alert);

	if ( ret>0 ) {
		syslog(LOG_NOTICE, "%s", p->buf);
	}else{
		mesg(M_CRIT,"syslog: error formatting log");
		return 0;
	}

	return 1;
}

int syslog_v(char *args, void **ptr)
{
	struct syslog_priv *priv;
	struct arg log_args[]={
		/* ident string */
		/* cons, nosait, perror, pid */
		/* facility */
		/* level */
		{NULL, ARGTYPE_NOP, NULL},
	};

	if ( !ipproto && !(ipproto=decode_proto("ip")) ) {
		mesg(M_ERR,"syslog: IP protocol must be loaded "
			"to use log output");
		return 0;
	}

	if ( !http_proto && !(http_proto=decode_proto("http")) ) {
		mesg(M_ERR,"syslog: HTTP protocol must be loaded "
			"to use log output");
		return 0;
	}

	if ( !(priv=calloc(1, sizeof(*priv))) )
		return 0;

	priv->buflen=4096;

	if ( args ) {
		switch ( args_parse(log_args, args, priv) ) {
		case -1:
			mesg(M_ERR,"syslog: parse error: %s", args);
		case 0: /* fall through */
			free(priv);
			return 0;
		default:
			break;
		}
	}

	if ( !(priv->buf=malloc(priv->buflen)) ) {
		mesg(M_ERR,"syslog: malloc(): %s", get_err());
		free(priv);
		return 0;
	}

	openlog("firestorm",
		LOG_CONS|LOG_NDELAY|LOG_PID,
		LOG_DAEMON);

	*ptr=priv;
	return 1;
}

int syslog_c(void *priv)
{
	closelog();

	if ( priv )
		free(priv);

	return 1;
}

struct target syslog_t[]={
	target_init("syslog", syslog_a, syslog_v, syslog_c,
		"sends alerts to syslog"),
	target_null()
};

int PLUGIN_TARGET (struct target_api *t)
{
	object_check(t);

	args_parse=t->args_parse;

	if ( !t->target_add(syslog_t) )
		return PLUGIN_ERR_FAIL;

	return PLUGIN_ERR_OK;
}

int PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	plugin_check(in, out);

	PLUGIN_ID("target.syslog", "Sends alerts to syslog");
	PLUGIN_VERSION(0,1);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@scaramanga.co.uk");
	PLUGIN_LICENSE("GPL");

	if ( !(decode_proto=in->import("decode.proto")) )
		return PLUGIN_ERR_OBJECT;

	return PLUGIN_ERR_OK;
}

int PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
